using System;
using Server.Items;
using Server.Network;

namespace Server.Scripts.Items.Weapons
{
	public class Fists : BaseWeapon
	{
		public static void Initialize()
		{
			BaseWeapon.Fists = new Fists();

			EventSink.DisarmRequest += new DisarmRequestEventHandler( EventSink_DisarmRequest );
			EventSink.StunRequest += new StunRequestEventHandler( EventSink_StunRequest );
		}

		public Fists() : base( 1, 8, 30, 0, 0, 1, SkillName.Wrestling, WeaponType.Fists, WeaponAnimation.Wrestle, 0, Layer.Invalid )
		{
			Visible = false;
			Movable = false;
		}

		public Fists( Serial serial ) : base( serial )
		{
		}

		public override double GetDefendSkillValue( Mobile attacker, Mobile defender )
		{
			double wresValue = defender.Skills[SkillName.Wrestling].Value;
			double anatValue = defender.Skills[SkillName.Anatomy].Value;
			double evalValue = defender.Skills[SkillName.EvalInt].Value;
			double incrValue = (anatValue + evalValue + 20.0) * 0.5;

			if ( incrValue > 120.0 )
				incrValue = 120.0;

			if ( wresValue > incrValue )
				return wresValue;
			else
				return incrValue;
		}

		public override void OnHit( Mobile attacker, Mobile defender )
		{
			base.PlaySwingAnimation( attacker );

			defender.AggressiveAction( attacker );
			defender.Damage( ComputeDamage( attacker, defender ) );

			if ( attacker.StunReady )
			{
				if ( attacker.CanBeginAction( typeof( Fists ) ) )
				{
					if ( attacker.Stam >= 5 )
					{
						attacker.Stam -= 5;

						if ( CheckMove( attacker, SkillName.Anatomy ) )
						{
							StartMoveDelay( attacker );

							attacker.StunReady = false;

							attacker.SendLocalizedMessage( 1004013 ); // You successfully stun your opponent!
							defender.SendLocalizedMessage( 1004014 ); // You have been stunned!

							defender.Freeze( TimeSpan.FromSeconds( 5.0 ) );
						}
						else
						{
							attacker.SendLocalizedMessage( 1004010 ); // You failed in your attempt to stun.
							defender.SendLocalizedMessage( 1004011 ); // Your opponent tried to stun you and failed.
						}
					}
					else
					{
						attacker.SendLocalizedMessage( 1004009 ); // You are too fatigued to attempt anything.
					}
				}
			}
			else if ( attacker.DisarmReady )
			{
				if ( attacker.CanBeginAction( typeof( Fists ) ) )
				{
					if ( attacker.Stam >= 5 )
					{
						Item toDisarm = defender.FindItemOnLayer( Layer.OneHanded );

						if ( toDisarm == null )
							toDisarm = defender.FindItemOnLayer( Layer.TwoHanded );

						Container pack = defender.Backpack;

						if ( pack == null || toDisarm == null )
						{
							attacker.SendLocalizedMessage( 1004001 ); // You cannot disarm your opponent.
						}
						else if ( CheckMove( attacker, SkillName.ArmsLore ) )
						{
							StartMoveDelay( attacker );

							attacker.Stam -= 5;
							attacker.DisarmReady = false;

							attacker.SendLocalizedMessage( 1004006 ); // You successfully disarm your opponent!
							defender.SendLocalizedMessage( 1004007 ); // You have been disarmed!

							pack.DropItem( toDisarm );
						}
						else
						{
							attacker.Stam -= 5;

							attacker.SendLocalizedMessage( 1004004 ); // You failed in your attempt to disarm.
							defender.SendLocalizedMessage( 1004005 ); // Your opponent tried to disarm you but failed.
						}
					}
					else
					{
						attacker.SendLocalizedMessage( 1004003 ); // You are too fatigued to attempt anything.
					}
				}
			}
		}

		public override void OnMiss( Mobile attacker, Mobile defender )
		{
			base.PlaySwingAnimation( attacker );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		/* Wrestling moves */

		private static bool CheckMove( Mobile m, SkillName other )
		{
			double wresValue = m.Skills[SkillName.Wrestling].Value;
			double scndValue = m.Skills[other].Value;

			/* 40% chance at 80, 80
			 * 50% chance at 100, 100
			 * 60% chance at 120, 120
			 */

			double chance = (wresValue + scndValue) / 400.0;

			return ( chance >= Utility.RandomDouble() );
		}

		private static bool HasFreeHands( Mobile m )
		{
			Item item = m.FindItemOnLayer( Layer.OneHanded );

			if ( item != null && !(item is Spellbook) )
				return false;

			return m.FindItemOnLayer( Layer.TwoHanded ) == null;
		}

		private static void EventSink_DisarmRequest( DisarmRequestEventArgs e )
		{
			Mobile m = e.Mobile;

			double armsValue = m.Skills[SkillName.ArmsLore].Value;
			double wresValue = m.Skills[SkillName.Wrestling].Value;

			if ( !HasFreeHands( m ) )
			{
				m.SendLocalizedMessage( 1004029 ); // You must have your hands free to attempt to disarm your opponent.
				m.DisarmReady = false;
			}
			else if ( armsValue >= 80.0 && wresValue >= 80.0 )
			{
				m.DisarmReady = !m.DisarmReady;
				m.SendLocalizedMessage( m.DisarmReady ? 1019013 : 1019014 );
			}
			else
			{
				m.SendLocalizedMessage( 1004002 ); // You are not skilled enough to disarm your opponent.
				m.DisarmReady = false;
			}
		}

		private static void EventSink_StunRequest( StunRequestEventArgs e )
		{
			Mobile m = e.Mobile;

			double anatValue = m.Skills[SkillName.Anatomy].Value;
			double wresValue = m.Skills[SkillName.Wrestling].Value;

			if ( !HasFreeHands( m ) )
			{
				m.SendLocalizedMessage( 1004031 ); // You must have your hands free to attempt to stun your opponent.
				m.StunReady = false;
			}
			else if ( anatValue >= 80.0 && wresValue >= 80.0 )
			{
				m.StunReady = !m.StunReady;
				m.SendLocalizedMessage( m.StunReady ? 1019011 : 1019012 );
			}
			else
			{
				m.SendLocalizedMessage( 1004008 ); // You are not skilled enough to stun your opponent.
				m.StunReady = false;
			}
		}

		private class MoveDelayTimer : Timer
		{
			private Mobile m_Mobile;

			public MoveDelayTimer( Mobile m ) : base( TimeSpan.FromSeconds( 10.0 ) )
			{
				m_Mobile = m;

				m_Mobile.BeginAction( typeof( Fists ) );
			}

			protected override void OnTick()
			{
				m_Mobile.EndAction( typeof( Fists ) );
			}
		}

		private static void StartMoveDelay( Mobile m )
		{
			new MoveDelayTimer( m ).Start();
		}
	}
}