using System;
using System.Collections;
using Server.Scripts.Misc;
using Server.Items;
using Server.Scripts.Items;
using Server.Scripts.Mobiles.Animals.Mounts;
using Server.Scripts.Items.Armor.Plate;
using Server.Scripts.Items.Armor.Helmets;
using Server.Scripts.Items.Weapons.PoleArms;

namespace Server.Scripts.Mobiles.People
{
	public class WarriorGuard : BaseGuard
	{
		private Timer m_AttackTimer, m_IdleTimer;

		private Mobile m_Focus;

		public WarriorGuard( Mobile target ) : base( target )
		{
			InitStats( 100, 125, 25 );
			Title = "the guard";

			Hue = Utility.RandomSkinHue();

			if ( Female = Utility.RandomBool() )
			{
				Body = 0x191;
				Name = NameList.RandomFemaleName();
			}
			else
			{
				Body = 0x190;
				Name = NameList.RandomMaleName();
			}

			new GrayHorse().Rider = this;

			AddItem( new PlateChest() );
			AddItem( new PlateArms() );
			AddItem( new PlateGloves() );
			AddItem( new PlateGorget() );
			AddItem( new PlateLegs() );
			AddItem( new PlateHelm() );

			Halberd weapon = new Halberd();

			weapon.Movable = false;
			weapon.Newbied = true;
			weapon.Crafter = this;
			weapon.Quality = WeaponQuality.Exceptional;

			AddItem( weapon );

			Container pack = new Backpack();

			pack.Movable = false;
			pack.Newbied = true;

			pack.DropItem( new Gold( 250, 500 ) );

			AddItem( pack );

			Skills[SkillName.Anatomy].Base = 120.0;
			Skills[SkillName.Tactics].Base = 120.0;
			Skills[SkillName.Swords].Base = 120.0;
			Skills[SkillName.MagicResist].Base = 120.0;

			this.NextCombatTime = DateTime.Now + TimeSpan.FromSeconds( 0.5 );
			this.Focus = target;
		}

		public WarriorGuard( Serial serial ) : base( serial )
		{
		}

		public override bool OnBeforeDeath()
		{
			if ( m_Focus != null && m_Focus.Alive )
				new AvengeTimer( m_Focus ).Start(); // If a guard dies, three more guards will spawn

			return base.OnBeforeDeath();
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public override Mobile Focus
		{
			get
			{
				return m_Focus;
			}
			set
			{
				if ( m_Focus != value )
				{
					if ( value != null )
						this.AggressiveAction( value );

					Combatant = value;

					if ( value != null )
					{
						Say( 500131 ); // Thou wilt regret thine actions, swine!
					}
					else if ( m_Focus != null )
					{
						Say( "Thou hast suffered thy punishment, scoundrel." );
					}

					m_Focus = value;

					if ( m_AttackTimer != null )
					{
						m_AttackTimer.Stop();
						m_AttackTimer = null;
					}

					if ( m_IdleTimer != null )
					{
						m_IdleTimer.Stop();
						m_IdleTimer = null;
					}

					if ( m_Focus != null )
					{
						m_AttackTimer = new AttackTimer( this );
						m_AttackTimer.Start();
					}
					else
					{
						m_IdleTimer = new IdleTimer( this );
						m_IdleTimer.Start();
					}
				}
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Focus );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Focus = reader.ReadMobile();

					if ( m_Focus != null )
					{
						m_AttackTimer = new AttackTimer( this );
						m_AttackTimer.Start();
					}
					else
					{
						m_IdleTimer = new IdleTimer( this );
						m_IdleTimer.Start();
					}

					break;
				}
			}
		}

		public override void OnAfterDelete()
		{
			if ( m_AttackTimer != null )
			{
				m_AttackTimer.Stop();
				m_AttackTimer = null;
			}

			if ( m_IdleTimer != null )
			{
				m_IdleTimer.Stop();
				m_IdleTimer = null;
			}

			base.OnAfterDelete();
		}

		private class AvengeTimer : Timer
		{
			private Mobile m_Focus;

			public AvengeTimer( Mobile focus ) : base( TimeSpan.FromSeconds( 5.0 ) )
			{
				m_Focus = focus;
			}

			protected override void OnTick()
			{
				BaseGuard.Spawn( m_Focus, 3, true );
			}
		}

		private class AttackTimer : Timer
		{
			private WarriorGuard m_Owner;
			private bool m_Shooting;

			public AttackTimer( WarriorGuard owner ) : base( TimeSpan.FromSeconds( 0.25 ), TimeSpan.FromSeconds( 0.1 ) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				// <?>
				m_Owner.Criminal = false;
				m_Owner.Kills = 0;
				// </?>

				Mobile target = m_Owner.Combatant;

				if ( target == null )
				{
					m_Owner.Focus = null;
				}
				else if ( !m_Owner.InRange( target, 20 ) )
				{
					m_Owner.Focus = null;
				}
				else if ( !m_Owner.InRange( target, 10 ) )
				{
					Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );

					m_Owner.Location = target.Location;

					Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );
					m_Owner.PlaySound( 0x1FE );
				}
				else if ( !m_Owner.InRange( target, 1 ) )
				{
					m_Owner.Move( m_Owner.GetDirectionTo( target ) | Direction.Running );
				}
			}
		}

		private class IdleTimer : Timer
		{
			private WarriorGuard m_Owner;
			private int m_Stage;

			public IdleTimer( WarriorGuard owner ) : base( TimeSpan.FromSeconds( 2.0 ), TimeSpan.FromSeconds( 2.5 ) )
			{
				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Deleted )
				{
					Stop();
					return;
				}

				Mobile target = m_Owner.Combatant;

				if ( target == null )
				{
					if ( (m_Stage++ % 4) == 0 || !m_Owner.Move( m_Owner.Direction ) )
						m_Owner.Direction = (Direction)Utility.Random( 8 );

					if ( m_Stage > 16 )
					{
						Effects.SendLocationEffect( m_Owner.Location, m_Owner.Map, 0x3728, 13 );
						m_Owner.PlaySound( 0x1FE );

						m_Owner.Delete();
					}
				}
				else
				{
					m_Owner.Focus = target;
				}
			}
		}
	}
}